/*
 * Tipo: Banco
 *
 * Marca Registrada (c) 2000 CiN
 * Todos os direitos reservados.
 *
 * Este software  confidencial e propriedade intelectual do 
 * CiN - Centro de Informtica. 
 * Voc no deve utilizar indevidamente este produto em desacordo com o 
 * contrato estabelecido com a empresa.
 */

package classesAbstratas;


/**
* Banco modela um banco com suas contas e operaes sobre as mesmas.
* 
*/
public class Banco {

    private int indice = 0; // nmero de contas cadastradas
    private Conta[] contas; // contas cadastradas no banco
    private final int MAX_CONTAS = 50; // nmero mximo de contas que podem ser cadastradas

    /**
     * Construtor da classe Banco.
     *
     */
    public Banco () {
        contas = new Conta[MAX_CONTAS];
        indice = 0;
    }

    /**
     * Este mtodo cadastra uma conta no banco.
     *
     * @param conta    conta a ser cadastrada
     *     
     */
    public void cadastrar(Conta conta) {
        if (indice < MAX_CONTAS) {
            contas[indice] = conta;
            indice = indice + 1;
        }
    }

    /**
     * Este mtodo procura uma conta neste banco, dada o nmero da conta procurada.
     *
     * @param numero    numero da conta a ser procurada
     *     
     * @exception CNEException  exceo lanada quando no existe uma conta com o nmero fornecido
     *
     */
    private Conta procurar(String numero) throws CNEException { 
        int i = 0; 
        boolean achou = false;
        Conta result;

        while ((! achou) & (i < indice)) {
            if (contas[i].getNumero().equals(numero)) {
                achou = true;
            }
            else {
                i = i + 1;
            }
        }

        if (achou == true) {
            result = contas[i];
        }
        else {
            throw new CNEException(numero);
        }
        return result;
    }

    /**
     * Este mtodo retorna o saldo da conta especificada.
     *
     * @param numero   O nmero da conta que se deseja o saldo.
     *
     * @exception CNEException  exceo lanada quando no existe uma conta com o nmero fornecido
     *     
     */
    public double retornaSaldo(String numero) throws CNEException {
        Conta c;

        c = procurar(numero);
        return c.getSaldo();
    }

    /**
     * Este mtodo debita o valor especificado na conta dada.
     *
     * @param numero   O nmero da conta que se deseja o dbito.
     * @param valor    O valor do dbito.     
     *
     * @exception CNEException  exceo lanada quando no existe uma conta com o nmero fornecido
     *     
     */
    public void debitar(String numero, double valor) throws CNEException {
        Conta c;

        c = this.procurar(numero); 
        c.debitar(valor);
    }

    /**
     * Este mtodo credita o valor especificado na conta dada.
     *
     * @param numero   O nmero da conta que se deseja o crdito.
     * @param valor    O valor do crdito.     
     *
     * @exception CNEException  exceo lanada quando no existe uma conta com o nmero fornecido
     *     
     */
    public void creditar(String numero, double valor) throws CNEException {
        Conta c;

        c = this.procurar(numero); 
        c.creditar(valor);
    }

    /**
     * Transfere um valor especificado entre duas contas.
     *
     * @param numeroDe   O nmero da conta a ser debitada.
     * @param numeroPara O nmero da conta a ser creditada.
     * @param valor      O valor a ser transferido.
     *
     * @exception CNEException  exceo lanada quando no existe uma conta com o nmero fornecido
     *          
     */
    public void transferir(String numeroDe, String numeroPara, double valor) throws CNEException {
        Conta contaDe, contaPara;

        contaDe = this.procurar(numeroDe);
        contaPara = this.procurar(numeroPara); 
        contaDe.debitar(valor);
        contaPara.creditar(valor);
    }

}